<?php

namespace v1\controllers;

use app\models\ServicioTipoServicio;
use common\data\Respuesta;
use common\rest\AuthController;
use Yii;
use yii\db\Expression;

class ServicioController extends AuthController {

  public $modelClass = "v1\models\Servicio";

  public function actionIndex() {
    $id = trim($this->req->get("id", ""));
    $buscar = trim($this->req->get("q", ""));
    $taller = trim($this->req->get("taller", ""));
    $vehiculo = trim($this->req->get("vehiculo", ""));
    $marca = trim($this->req->get("marca", ""));
    $inicio = trim($this->req->get("inicio", ""));
    $fin = trim($this->req->get("fin", ""));
    $query = $this->queryInicial
      ->innerJoin("Vehiculo", "{{Vehiculo}}.id = {{Servicio}}.[[idVehiculo]]")
      ->innerJoin("Marca", "{{Marca}}.id = {{Vehiculo}}.[[idMarca]]")
      ->innerJoin("Taller", "{{Taller}}.id = {{Servicio}}.[[idTaller]]");


    if ($id !== "") {
      $query->andWhere(["{{Servicio}}.id" => $id]);
    }

    if ($taller !== "") {
      $query->andWhere(["{{Taller}}.id" => $taller]);
    }

    if ($marca !== "") {
      $query->andWhere(["{{Marca}}.id" => $marca]);
    }

    if ($vehiculo !== "") {
      $query->andWhere(["{{Vehiculo}}.id" => $vehiculo]);
    }
    /* if($marca !== "") {
      $query->andWhere(["{{Marca}}.id" => $marca]);
  }*/
    if ($inicio !== "" && $fin !== "") {
      $query->andWhere(["between", "[[fechaServicio]]", $inicio, $fin]);
    }
    if ($buscar) {

      $query->andWhere([
        "OR",
        ["ilike", "{{Vehiculo}}.nombre", $buscar]
      ]);
    }

    return new Respuesta($query, $this->limite, $this->pagina, $this->ordenar);
  }

  public function actionGuardar() {
    $id = trim($this->req->getBodyParam("id", ""));
    $tipoServicio = $this->req->getBodyParam("tipoServicio", []);
    $modelo = null;

    $transaccion = \Yii::$app->db->beginTransaction();
    try {

      if ($id !== "") {
        $modelo = $this->modelClass::findOne($id);
      }
      if ($modelo === null) {
        $modelo = new $this->modelClass();
        $modelo->uuid();
        $modelo->creado = new Expression('now()');
      } else {
        $modelo->modificado = new Expression('now()');
      }

      $modelo->load($this->req->getBodyParams(), '');
      if (!$modelo->save()) {
        return (new Respuesta($modelo))
          ->mensaje("Hubo un problema al guardar el Servicio");
      }

      ServicioTipoServicio::deleteAll(['id' => $modelo->id]);

      foreach ($tipoServicio as $tipoServicioItem) {

        $tipoServicios = ServicioTipoServicio::findOne([
          'idServicio' => $modelo->id,
          'idTipoServicio' => $tipoServicioItem['idTipoServicio']
        ]);

        if ($tipoServicios === null) {
          $tipoServicios = new ServicioTipoServicio();
          $tipoServicios->uuid();
          $tipoServicios->idServicio = $modelo->id;
          $tipoServicios->idTipoServicio = $tipoServicioItem['idTipoServicio'];
          $tipoServicios->creado = new Expression('now()');
        } else {
          $tipoServicios->modificado = new Expression('now()');
        }

        $tipoServicios->nombre = $tipoServicioItem['nombre'];
        $tipoServicios->costo = $tipoServicioItem['costo'];

        if (!$tipoServicios->save()) {
          return (new Respuesta($tipoServicios))
            ->mensaje("Hubo un problema al guardar el Tipo de Servicio");
        }
      }

      $transaccion->commit();
      $modelo->refresh();
      return (new Respuesta($modelo))
        ->mensaje("Servicio ha sido guardado de manera exitosa");
    } catch (\Exception $e) {
      $transaccion->rollBack();
      return (new Respuesta($modelo))
        ->esError()
        ->mensaje($e->getMessage());
    }
  }

  public function actionEliminar() {
    $id = trim($this->req->getBodyParam("id", ""));
    $modelo = null;

    if ($id !== "") {
      $modelo = $this->modelClass::findOne(["id" => $id]);
    }
    if ($modelo === null) {
      return (new Respuesta())
        ->esError()
        ->mensaje("Servicio no encontrado");
    }
    $modelo->eliminado = new Expression('now()');
    if (!$modelo->save()) {
      return (new Respuesta($modelo))
        ->mensaje("No se pudo eliminar Servicio");
    }

    return (new Respuesta())
      ->mensaje("Servicio eliminada");
  }
}
